/* USER CODE BEGIN Header */
/* you can remove the copyright */

/*
 *  Copyright 2020-2023 Yuntu Microelectronics co.,ltd
 *  All rights reserved.
 * 
 *  YUNTU Confidential. This software is owned or controlled by YUNTU and may only be
 *  used strictly in accordance with the applicable license terms. By expressly
 *  accepting such terms or by downloading, installing, activating and/or otherwise
 *  using the software, you are agreeing that you have read, and that you agree to
 *  comply with and are bound by, such license terms. If you do not agree to be
 *  bound by the applicable license terms, then you may not retain, install,
 *  activate or otherwise use the software. The production use license in
 *  Section 2.3 is expressly granted for this software.
 * 
 * @file main.c
 * @brief 
 * 
 */

/* USER CODE END Header */
#include "sdk_project_config.h"
/* Includes ------------------------------------------------------------------*/

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "printf.h"
#include "psa/crypto.h"
#include "mbedtls/platform.h"
#include "mbedtls/bignum.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */
/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
#define PRINTF_UART (2) /* UART instance for PRINTF */
#define AES_TEST_BUFFER_LENGTH 16
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */
/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
/* USER CODE BEGIN PV */
static const unsigned char aes_test_key[16] = {
    0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
    0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f
};

static const unsigned char aes_test_plaintext[16] = {
    0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
    0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff
};

static unsigned char aes_test_ciphertext[16];
/* USER CODE END PV */

/* Private function declare --------------------------------------------------*/
/* USER CODE BEGIN PFDC */
static void print_buf(const char* title, const unsigned char* buf, size_t len);
static int aes_encrypt_test(void);
static int bignum_mul_test(void);
/* USER CODE END PFDC */
static void Board_Init(void);

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */
static void print_buf(const char* title, const unsigned char* buf, size_t len)
{
    PRINTF("%s: ", title);
    for(size_t i = 0; i < len; i++)
        PRINTF("%02x", buf[i]);
    PRINTF("\n");
}

static int aes_encrypt_test(void)
{
    psa_status_t status;
    psa_key_attributes_t attributes = PSA_KEY_ATTRIBUTES_INIT;
    psa_key_id_t key_id = 0;

    // 设置密钥属性
    psa_set_key_usage_flags(&attributes, PSA_KEY_USAGE_ENCRYPT);
    psa_set_key_algorithm(&attributes, PSA_ALG_ECB_NO_PADDING);
    psa_set_key_type(&attributes, PSA_KEY_TYPE_AES);
    psa_set_key_bits(&attributes, 128);

    // 导入密钥
    status = psa_import_key(&attributes,
                           aes_test_key,
                           sizeof(aes_test_key),
                           &key_id);
    if (status != PSA_SUCCESS) {
        PRINTF("Key import failed! (status = %d)\n", status);
        return -1;
    }

    // 执行加密
    size_t output_length;
    status = psa_cipher_encrypt(key_id,
                              PSA_ALG_ECB_NO_PADDING,
                              aes_test_plaintext,
                              sizeof(aes_test_plaintext),
                              aes_test_ciphertext,
                              sizeof(aes_test_ciphertext),
                              &output_length);

    if (status != PSA_SUCCESS) {
        PRINTF("Encryption failed! (status = %d)\n", status);
        psa_destroy_key(key_id);
        return -1;
    }

    print_buf("Key", aes_test_key, 16);
    print_buf("Plaintext", aes_test_plaintext, 16);
    print_buf("Ciphertext", aes_test_ciphertext, 16);

    // 清理密钥
    psa_destroy_key(key_id);
    return 0;
}

static int bignum_mul_test(void)
{
    mbedtls_mpi A, B, R;
    int ret;

    mbedtls_mpi_init(&A);
    mbedtls_mpi_init(&B);
    mbedtls_mpi_init(&R);

    // 设置两个大数: A = 123456789, B = 987654321
    MBEDTLS_MPI_CHK(mbedtls_mpi_read_string(&A, 10, "123456789"));
    MBEDTLS_MPI_CHK(mbedtls_mpi_read_string(&B, 10, "987654321"));

    // 执行乘法运算: R = A * B
    MBEDTLS_MPI_CHK(mbedtls_mpi_mul_mpi(&R, &A, &B));

    // 打印结果
    char buf[256];
    size_t n;
    
    PRINTF("\nBignum multiplication test:\n");
    
    mbedtls_mpi_write_string(&A, 10, buf, sizeof(buf), &n);
    PRINTF("A = %s\n", buf);
    
    mbedtls_mpi_write_string(&B, 10, buf, sizeof(buf), &n);
    PRINTF("B = %s\n", buf);
    
    mbedtls_mpi_write_string(&R, 10, buf, sizeof(buf), &n);
    PRINTF("A * B = %s\n", buf);

cleanup:
    mbedtls_mpi_free(&A);
    mbedtls_mpi_free(&B);
    mbedtls_mpi_free(&R);
    
    return ret;
}
/* USER CODE END 0 */


/**
 * @brief  The application entry point.
 * @retval int
 */
int main(void)
{
    /* USER CODE BEGIN 1 */
    status_t status = STATUS_SUCCESS;
    mbedtls_config_init();
    // 初始化 PSA Crypto
    psa_status_t psa_status = psa_crypto_init();
    if (psa_status != PSA_SUCCESS) {
        PRINTF("PSA init failed! (status = %d)\n", psa_status);
        return STATUS_ERROR;
    }
    /* USER CODE END 1 */ 
    Board_Init();
    /* USER CODE BEGIN 2 */
    PRINTF("\nMbedTLS AES Encryption Test:\n");
    if(aes_encrypt_test() == 0)
    {
        PRINTF("AES encryption test succeeded!\n");
    }
    else
    {
        PRINTF("AES encryption test failed!\n");
        status = STATUS_ERROR;
    }

    // 大数乘法测试
    if(bignum_mul_test() == 0)
    {
        PRINTF("Bignum multiplication test succeeded!\n");
    }
    else
    {
        PRINTF("Bignum multiplication test failed!\n");
        status = STATUS_ERROR;
    }

    /* USER CODE END 2 */

    /* Infinite loop */
    /* USER CODE BEGIN WHILE */
    while (1)
    {
        if (status != STATUS_SUCCESS)
        {
            break;
        }
        /* USER CODE END WHILE */
        /* USER CODE BEGIN 3 */
    }
    return status;
    /* USER CODE END 3 */
}

static void Board_Init(void)
{
    CLOCK_SYS_Init(g_clockManConfigsArr,CLOCK_MANAGER_CONFIG_CNT,g_clockManCallbacksArr,CLOCK_MANAGER_CALLBACK_CNT);
    CLOCK_SYS_UpdateConfiguration(CLOCK_MANAGER_ACTIVE_INDEX,CLOCK_MANAGER_POLICY_AGREEMENT);
    PINS_DRV_Init(NUM_OF_CONFIGURED_PINS0,g_pin_mux_InitConfigArr0);
    UTILITY_PRINT_Init();
}

/* USER CODE BEGIN 4 */
/* USER CODE END 4 */
